BIN_DIR		= bin
SRC_DIR		= src
LIB_DIR		= lib
LIB_DIR_DEST	= /usr/local/lib
INCLUDE_DIR	= include
INCLUDE_DIR_DEST= /usr/local/include
CPP             = CC 
MPICPP		= tmCC -lmpi
AR              = CC -xar -o
CPPFLAGS        = -I./$(INCLUDE_DIR) -`fpversion -foption`
LFLAGS          = -lm -ltrng -L./$(LIB_DIR)
LIB             = $(LIB_DIR)/libtrng.a
ALL_EXE		= plausibility profile diehard_file exception_demo template_demo copy_demo boolean_dist_test uniform_dist_test normal_dist_test exp_dist_test laplace_dist_test tent_dist_test tent_dist_test gamma_dist_test beta_dist_test chi_squared_dist_test Student_t_dist_test binomial_dist_test poisson_dist_test geometric_dist_test rejection_test bitequidisttest collisiontest couponcollectorstest gaptest max_of_t_test partitiontest permutationtest runtest serialcorrelationtest serialtest sitetest spectral integral ising2dwolff_queue ising2dwolff_stack randomwalktest pi pi_advanced
ALL_SRC         = $(addprefix $(SRC_DIR)/, trng.cc trnglib.cc $(addsuffix .cc, $(ALL_EXE)))
ALL_OBJECTS     = ${ALL_SRC:.cc=.o}

lib: $(LIB)

install: lib
	install -d $(LIB_DIR_DEST)
	install -f $(LIB_DIR_DEST) $(LIB)
	install -d $(INCLUDE_DIR_DEST)
        install -f $(INCLUDE_DIR_DEST) $(INCLUDE_DIR)/trng.h
	install -f $(INCLUDE_DIR_DEST) $(INCLUDE_DIR)/trnglib.h

all: lib examples

examples: $(addprefix $(BIN_DIR)/, $(ALL_EXE))

$(ALL_OBJECTS): $(INCLUDE_DIR)/trng.h

$(SRC_DIR)/trnglib.o: $(INCLUDE_DIR)/trnglib.h

$(SRC_DIR)/trng.o: $(INCLUDE_DIR)/trng.h


# TRNG lib

$(LIB): $(SRC_DIR)/trnglib.o $(SRC_DIR)/trng.o
	$(AR) $(LIB) $(SRC_DIR)/trnglib.o $(SRC_DIR)/trng.o
	$(RM) -r $(SRC_DIR)/SunWS_cache

# test sequence splitting and leapfrog method

$(BIN_DIR)/plausibility: $(LIB) $(SRC_DIR)/plausibility.o
	$(CPP) -o $(BIN_DIR)/plausibility $(SRC_DIR)/plausibility.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache


# profiling

$(BIN_DIR)/profile: $(LIB) $(SRC_DIR)/profile.o
	$(CPP) -o $(BIN_DIR)/profile $(SRC_DIR)/profile.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache


# generate file for diehard test

$(BIN_DIR)/diehard_file: $(LIB) $(SRC_DIR)/diehard_file.o
	$(CPP) -o $(BIN_DIR)/diehard_file $(SRC_DIR)/diehard_file.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache


# demos for exception handling and so

$(BIN_DIR)/exception_demo: $(LIB) $(SRC_DIR)/exception_demo.o
	$(CPP) -o $(BIN_DIR)/exception_demo $(SRC_DIR)/exception_demo.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/template_demo: $(LIB) $(SRC_DIR)/template_demo.o
	$(CPP) -o $(BIN_DIR)/template_demo $(SRC_DIR)/template_demo.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/copy_demo: $(LIB) $(SRC_DIR)/copy_demo.o
	$(CPP) -o $(BIN_DIR)/copy_demo $(SRC_DIR)/copy_demo.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache


# distribution tests

$(BIN_DIR)/boolean_dist_test: $(LIB) $(SRC_DIR)/boolean_dist_test.o
	$(CPP) -o $(BIN_DIR)/boolean_dist_test $(SRC_DIR)/boolean_dist_test.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/uniform_dist_test: $(LIB) $(SRC_DIR)/uniform_dist_test.o
	$(CPP) -o $(BIN_DIR)/uniform_dist_test $(SRC_DIR)/uniform_dist_test.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/normal_dist_test: $(LIB) $(SRC_DIR)/normal_dist_test.o
	$(CPP) -o $(BIN_DIR)/normal_dist_test $(SRC_DIR)/normal_dist_test.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/exp_dist_test: $(LIB) $(SRC_DIR)/exp_dist_test.o
	$(CPP) -o $(BIN_DIR)/exp_dist_test $(SRC_DIR)/exp_dist_test.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/laplace_dist_test: $(LIB) $(SRC_DIR)/laplace_dist_test.o
	$(CPP) -o $(BIN_DIR)/laplace_dist_test $(SRC_DIR)/laplace_dist_test.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/tent_dist_test: $(LIB) $(SRC_DIR)/tent_dist_test.o
	$(CPP) -o $(BIN_DIR)/tent_dist_test $(SRC_DIR)/tent_dist_test.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/gamma_dist_test: $(LIB) $(SRC_DIR)/gamma_dist_test.o
	$(CPP) -o $(BIN_DIR)/gamma_dist_test $(SRC_DIR)/gamma_dist_test.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/beta_dist_test: $(LIB) $(SRC_DIR)/beta_dist_test.o
	$(CPP) -o $(BIN_DIR)/beta_dist_test $(SRC_DIR)/beta_dist_test.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/chi_squared_dist_test: $(LIB) $(SRC_DIR)/chi_squared_dist_test.o
	$(CPP) -o $(BIN_DIR)/chi_squared_dist_test $(SRC_DIR)/chi_squared_dist_test.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/Student_t_dist_test: $(LIB) $(SRC_DIR)/Student_t_dist_test.o
	$(CPP) -o $(BIN_DIR)/Student_t_dist_test $(SRC_DIR)/Student_t_dist_test.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/binomial_dist_test: $(LIB) $(SRC_DIR)/binomial_dist_test.o
	$(CPP) -o $(BIN_DIR)/binomial_dist_test $(SRC_DIR)/binomial_dist_test.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/poisson_dist_test: $(LIB) $(SRC_DIR)/poisson_dist_test.o
	$(CPP) -o $(BIN_DIR)/poisson_dist_test $(SRC_DIR)/poisson_dist_test.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/geometric_dist_test: $(LIB) $(SRC_DIR)/geometric_dist_test.o
	$(CPP) -o $(BIN_DIR)/geometric_dist_test $(SRC_DIR)/geometric_dist_test.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/rejection_test: $(LIB) $(SRC_DIR)/rejection_test.o
	$(CPP) -o $(BIN_DIR)/rejection_test $(SRC_DIR)/rejection_test.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache


# tests from Knuth

$(BIN_DIR)/bitequidisttest: $(LIB) $(SRC_DIR)/bitequidisttest.o
	$(CPP) -o $(BIN_DIR)/bitequidisttest $(SRC_DIR)/bitequidisttest.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/collisiontest: $(LIB) $(SRC_DIR)/collisiontest.o
	$(CPP) -o $(BIN_DIR)/collisiontest $(SRC_DIR)/collisiontest.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/couponcollectorstest: $(LIB) $(SRC_DIR)/couponcollectorstest.o
	$(CPP) -o $(BIN_DIR)/couponcollectorstest $(SRC_DIR)/couponcollectorstest.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/gaptest: $(LIB) $(SRC_DIR)/gaptest.o
	$(CPP) -o $(BIN_DIR)/gaptest $(SRC_DIR)/gaptest.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/max_of_t_test: $(LIB) $(SRC_DIR)/max_of_t_test.o
	$(CPP) -o $(BIN_DIR)/max_of_t_test $(SRC_DIR)/max_of_t_test.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/partitiontest: $(LIB) $(SRC_DIR)/partitiontest.o
	$(CPP) -o $(BIN_DIR)/partitiontest $(SRC_DIR)/partitiontest.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/permutationtest: $(LIB) $(SRC_DIR)/permutationtest.o
	$(CPP) -o $(BIN_DIR)/permutationtest $(SRC_DIR)/permutationtest.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/runtest: $(LIB) $(SRC_DIR)/runtest.o
	$(CPP) -o $(BIN_DIR)/runtest $(SRC_DIR)/runtest.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/serialcorrelationtest: $(LIB) $(SRC_DIR)/serialcorrelationtest.o
	$(CPP) -o $(BIN_DIR)/serialcorrelationtest $(SRC_DIR)/serialcorrelationtest.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/serialtest: $(LIB) $(SRC_DIR)/serialtest.o
	$(CPP) -o $(BIN_DIR)/serialtest $(SRC_DIR)/serialtest.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache


# other tests

$(BIN_DIR)/sitetest: $(LIB) $(SRC_DIR)/sitetest.o
	$(CPP) -o $(BIN_DIR)/sitetest $(SRC_DIR)/sitetest.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/spectral: $(SRC_DIR)/spectral.o
	$(CPP) -o $(BIN_DIR)/spectral $(SRC_DIR)/spectral.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

# monte carlo integration

$(BIN_DIR)/integral: $(LIB) $(SRC_DIR)/integral.o
	$(CPP) -o $(BIN_DIR)/integral $(SRC_DIR)/integral.o $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache


# ising model and random walk

$(BIN_DIR)/ising2dwolff_queue: $(LIB) $(INCLUDE_DIR)/trng.h $(SRC_DIR)/ising2dwolff_queue.cc
	$(MPICPP) $(CPPFLAGS) -o $(BIN_DIR)/ising2dwolff_queue $(SRC_DIR)/ising2dwolff_queue.cc $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/ising2dwolff_stack: $(LIB) $(INCLUDE_DIR)/trng.h $(SRC_DIR)/ising2dwolff_stack.cc
	$(MPICPP) $(CPPFLAGS) -o $(BIN_DIR)/ising2dwolff_stack $(SRC_DIR)/ising2dwolff_stack.cc $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/randomwalktest: $(LIB) $(INCLUDE_DIR)/trng.h $(SRC_DIR)/randomwalktest.cc
	$(MPICPP) $(CPPFLAGS) -o $(BIN_DIR)/randomwalktest $(SRC_DIR)/randomwalktest.cc $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache


# examples

$(BIN_DIR)/pi: $(LIB) $(INCLUDE_DIR)/trng.h $(SRC_DIR)/pi.cc
	$(MPICPP) $(CPPFLAGS) -o $(BIN_DIR)/pi $(SRC_DIR)/pi.cc $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache

$(BIN_DIR)/pi_advanced: $(LIB) $(INCLUDE_DIR)/trng.h $(SRC_DIR)/pi_advanced.cc
	$(MPICPP) $(CPPFLAGS) -o $(BIN_DIR)/pi_advanced $(SRC_DIR)/pi_advanced.cc $(LFLAGS)
	$(RM) -r $(SRC_DIR)/SunWS_cache


.cc.o:
	$(CPP) $(CPPFLAGS) -c -o $@ $<

clean:
	$(RM) $(ALL_OBJECTS) $(addprefix $(BIN_DIR)/, $(ALL_EXE)) $(LIB) $(SRC_DIR)/*.cc~ $(INCLUDE_DIR)/*.h~
	$(RM) -r $(SRC_DIR)/SunWS_cache
