// ---------------------------------------------------------------------
// Time-stamp: <Samstag, 09.02.2002,  1:27:00; edited by heiko>
// 
// Tina's random number generators TRNG
//
// useful functions for Tina's random number generators 
//
// Copyright (C) 2001, 2002 Heiko Bauke
//
// heiko.bauke@student.uni-magdeburg.de
//
// TRNG is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation. This program
// is distributed WITHOUT ANY WARRANTY; without even the implied
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// ---------------------------------------------------------------------

#if !defined TRNGLIB_H

#define TRNGLIB_H

#include <vector>

namespace TRNG {

  //! class for %error handling.
  /*!
    This class is thrown, if an %error occurs.
  */
  class error {
  public:
    //! %error message
    const char * message;
    //! constructor
    error(const char * m) : message(m) {};
  };

  const char * version(void);
  long modulo_invers(long, long);
  void gauss(std::vector<long> &, std::vector<long> &, long);
  void matrix_mult(const std::vector<long> &, const std::vector<long> &,
		   std::vector<long> &, long);
  void matrix_vec_mult(const std::vector<long> &, const std::vector<long> &, 
		       std::vector<long> &, long);

  double Gamma(double);
  double ln_Gamma(double);
  double Gamma_P(double, double);
  double Gamma_Q(double, double);
  double incomp_Gamma(double, double);
  double comp_incomp_Gamma(double, double);
  double Gamma_ser(double, double);
  double Gamma_cf(double, double);
  double ln_factorial(long);
  long binomial_coeff(long, long);
  double errf(double);
  double chi_square_test(const std::vector<double> &, 
			 const std::vector<double> &);
  double chi_square_prob(double, long);
  double Stirling_num2(long, long);
  double Student_t(double, long, bool=true);
  long find_interval(const std::vector<double> &, const double);

  double uniform_pdf(double);
  double uniform_pdf(double, double, double);
  double uniformco_pdf(double);
  double uniformco_pdf(double, double, double);
  double uniformcc_pdf(double);
  double uniformcc_pdf(double, double, double);
  double uniformoc_pdf(double);
  double uniformoc_pdf(double, double, double);
  double uniformoo_pdf(double);
  double uniformoo_pdf(double, double, double);
  double normal_dist_pdf(double, double, double);
  double exp_dist_pdf(double, double);
  double laplace_dist_pdf(double, double);
  double tent_dist_pdf(double, double);
  double Gamma_dist_pdf(double, double, double);
  double Beta_dist_pdf(double, double, double);
  double chi_square_dist_pdf(double, double);
  double Student_t_dist_pdf(double, double);
  double binomial_dist_pdf(long, long, double);
  double poisson_dist_pdf(long, double);
  double geometric_dist_pdf(long, double);
}

#endif


